#include "ppc.h"
#include "framebuffer.h"
#include <math.h>

PPC::PPC(int _w, int _h, float hfov) : w(_w), h(_h) {

  C = V3(0.0f, 0.0f, 0.0f);
  a = V3(1.0f, 0.0f, 0.0f);
  b = V3(0.0f, -1.0f, 0.0f);
  c[0] = -(float)w/2.0f;
  c[1] = +(float)h/2.0f;
  float tan2 = tanf(hfov/180.0f*3.1415f/2.0f);
  c[2] = -(float)w/2.0f/tan2;

}

bool PPC::Project(V3 P, V3 &projP) {

  M3x3 camM;
  camM.SetColumn(0, a);
  camM.SetColumn(1, b);
  camM.SetColumn(2, c);
  M3x3 invCamM = camM.Invert();
  V3 Q = invCamM*(P-C);

  if (Q[2] < 0.0f)
    return false; // behind head

  projP[0] = Q[0]/Q[2];
  projP[1] = Q[1]/Q[2];
  projP[2] = 1.0f/Q[2];

  return true;

}


void PPC::Visualize(PPC *ppc, float d, FrameBuffer *fb) {

  float scf = d / GetF();
  int psize = 5;
  unsigned int color = 0xFF000000;
  V3 colorv; colorv.SetFromColor(color);
  fb->DrawPoint3D(C, psize, color, ppc);
  V3 a1 = a*scf;
  V3 b1 = b*scf;
  V3 c1 = c*scf;
  fb->DrawSegment3D(C, C+c1, colorv, colorv, ppc);
  fb->DrawSegment3D(C+c1, C+c1+a1*(float)w, colorv, colorv, ppc);
  fb->DrawSegment3D(C+c1+a1*(float)w, C+c1+a1*(float)w+b1*(float)h, 
    colorv, colorv, ppc);
  fb->DrawSegment3D(C+c1+a1*(float)w+b1*(float)h, 
    C+c1+b1*(float)h, 
    colorv, colorv, ppc);
  fb->DrawSegment3D(C+c1+b1*(float)h, C+c1,
    colorv, colorv, ppc);

}

void PPC::Translate(V3 trans) {

  C = C+trans;

}

void PPC::TranslateFW(float step) {

  V3 vd = GetVD();
  Translate(vd*step);

}

V3 PPC::GetVD() {

  V3 ret = a^b;
  ret = ret.Normalized();
  return ret;

}

float PPC::GetF() {

  return c*GetVD();

}
